﻿using HtmlAgilityPack;
using Microsoft.Extensions.Logging;
using NotingTest;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Media;
using Telerik.Windows.Documents.Model;

namespace INV.Service
{
    public class ServiceHelper
    {
        private readonly ILogger<ServiceHelper> logger;

        public ServiceHelper(ILogger<ServiceHelper> logger)
        {
            this.logger = logger;
        }

        /// <summary>
        /// Convert Telerik Document Flow types
        /// </summary>
        /// <param name="content">string content of document</param>
        /// <param name="sourceType">type of current document</param>
        /// <param name="targetType">target type of resultent doucment</param>
        /// <returns>return string content of target type</returns>
        public string ConvertDocumentContent(string content, DocumentContentType? sourceType, DocumentContentType targetType, object logData)
        {
            if (string.IsNullOrEmpty(content)) return string.Empty;

            if (targetType == DocumentContentType.HTML && (sourceType ?? DocumentContentType.HTML) == DocumentContentType.RTF)
            {
                Telerik.Windows.Documents.FormatProviders.Rtf.RtfFormatProvider rtfProvider = new Telerik.Windows.Documents.FormatProviders.Rtf.RtfFormatProvider();
                Telerik.Windows.Documents.FormatProviders.Html.HtmlFormatProvider htmlProvider = new Telerik.Windows.Documents.FormatProviders.Html.HtmlFormatProvider();

                RadDocument document = null;
                try
                {
                    document = rtfProvider.Import(content);
                }
                catch (Exception ex)
                {
                    document = PrepareErrorMessageForDocument(ex);
                    using (Serilog.Context.LogContext.PushProperty("Data", logData, true))
                    {
                        logger.LogError(ex, ex.Message);
                    }
                }
                return htmlProvider.Export(document);

            }
            // convert from html to rtf
            else if (targetType == DocumentContentType.RTF && (sourceType ?? DocumentContentType.HTML) == DocumentContentType.HTML)
            {
                Telerik.Windows.Documents.FormatProviders.Html.HtmlFormatProvider htmlProvider = new Telerik.Windows.Documents.FormatProviders.Html.HtmlFormatProvider();
                Telerik.Windows.Documents.FormatProviders.Rtf.RtfFormatProvider rtfProvider = new Telerik.Windows.Documents.FormatProviders.Rtf.RtfFormatProvider();

                RadDocument document = null;

                try
                {
                    content = content.Replace("ListParagraph(numbered(a))", "");
                    HtmlDocument htmlDocument = new HtmlDocument();
                    htmlDocument.LoadHtml(content);
                    RemoveEmptyNodesFromHtml(htmlDocument.DocumentNode);

                    using (var stream = new MemoryStream())
                    {
                        htmlDocument.Save(stream);
                        stream.Seek(0, SeekOrigin.Begin);
                        content = Encoding.Default.GetString(stream.ToArray());
                    }

                    // Remove symbol font
                    string regex = @"(?i)font-family:\s*['\""]?Symbol(?-i)['\""]?;?";
                    content = Regex.Replace(content, regex, "");

                    document = htmlProvider.Import(content);

                    return rtfProvider.Export(document);
                }
                catch (Exception ex)
                {
                    document = PrepareErrorMessageForDocument(ex);
                    using (Serilog.Context.LogContext.PushProperty("Data", logData, true))
                    {
                        logger.LogError(ex, ex.Message);
                    }
                    return rtfProvider.Export(document);
                }

            }
            return content;
        }

        private void RemoveEmptyNodesFromHtml(HtmlNode containerNode)
        {
            List<string> nodesToRemove = new List<string> { "a", "table" };
            if (nodesToRemove.Contains(containerNode.Name) && string.IsNullOrEmpty(containerNode.InnerText))
            {
                containerNode.Remove();
            }
            else
            {
                for (int i = containerNode.ChildNodes.Count - 1; i >= 0; i--)
                {
                    RemoveEmptyNodesFromHtml(containerNode.ChildNodes[i]);
                }
            }
        }

        private RadDocument PrepareErrorMessageForDocument(Exception ex)
        {
            RadDocument document = new RadDocument();
            Section section = new Section();
            Paragraph paragraph = new Paragraph();

            var run = new Span();
            run.Text = "Para is not supported";
            run.FontWeight = System.Windows.FontWeights.Bold;
            run.FontSize = 16;
            run.ForeColor = Colors.Red;
            paragraph.Inlines.Add(run);

            Break br = new Break();
            paragraph.Inlines.Add(br);


            var run_excep = new Span();
            run_excep.Text = ex.Message;
            paragraph.Inlines.Add(run_excep);


            section.Blocks.Add(paragraph);
            document.Sections.Add(section);

            return document;
        }
    }
}
